@@ -3,7 +3,7 @@ from __future__ import absolute_import
 from __future__ import division
 
 
-from compas.datastructures.mesh.core import mesh_split_face
+from .operations import mesh_split_face
 
 
 __all__ = [
@@ -26,13 +26,3 @@ def mesh_quads_to_triangles(mesh, check_angles=False):
             # mesh.facedata[t2] = attr.copy()
             if fkey in mesh.facedata:
                 del mesh.facedata[fkey]
-
-
-# ==============================================================================
-# Main
-# ==============================================================================
-
-if __name__ == "__main__":
-
-    import doctest
-    doctest.testmod(globs=globals())
