@@ -15,11 +15,3 @@ class CollectionNumpy(object):
 
     def transformed(self, X):
         raise NotImplementedError
-
-
-# ==============================================================================
-# Main
-# ==============================================================================
-
-if __name__ == '__main__':
-    pass
