@@ -15,10 +15,7 @@ from compas.geometry import Circle
 from compas.geometry import Frame
 from compas.geometry import Plane
 
-from compas.geometry.shapes._shape import Shape
-
-
-__all__ = ['Cone']
+from ._shape import Shape
 
 
 class Cone(Shape):
@@ -56,10 +53,25 @@ class Cone(Shape):
 
     """
 
+    @property
+    def DATASCHEMA(self):
+        import schema
+        return schema.Schema({
+            'circle': {
+                'plane': Plane.DATASCHEMA.fget(None),
+                'radius': schema.And(float, lambda x: x > 0)
+            },
+            'height': schema.And(float, lambda x: x > 0)
+        })
+
+    @property
+    def JSONSCHEMANAME(self):
+        return 'cone'
+
     __slots__ = ['_circle', '_height']
 
-    def __init__(self, circle, height):
-        super(Cone, self).__init__()
+    def __init__(self, circle, height, **kwargs):
+        super(Cone, self).__init__(**kwargs)
         self._circle = None
         self._height = None
         self.circle = circle
@@ -89,7 +101,7 @@ class Cone(Shape):
 
     @plane.setter
     def plane(self, plane):
-        self.circle.plane = Plane(plane[0], plane[1])
+        self.circle.plane = Plane(*plane)
 
     @property
     def circle(self):
@@ -98,7 +110,7 @@ class Cone(Shape):
 
     @circle.setter
     def circle(self, circle):
-        self._circle = Circle(circle[0], circle[1])
+        self._circle = Circle(*circle)
 
     @property
     def radius(self):
@@ -204,22 +216,22 @@ class Cone(Shape):
         >>> cone = Cone.from_data(data)
 
         """
-        cone = cls(Circle(Plane.worldXY(), 1), 1)
-        cone.data = data
+        cone = cls(Circle.from_data(data['circle']), data['height'])
         return cone
 
     # ==========================================================================
     # methods
     # ==========================================================================
 
-    def to_vertices_and_faces(self, u=10):
+    def to_vertices_and_faces(self, u=16, triangulated=False):
         """Returns a list of vertices and faces.
 
         Parameters
         ----------
         u : int, optional
             Number of faces in the "u" direction.
-            Default is ``10``.
+        triangulated: bool, optional
+            Flag indicating that the faces have to be triangulated.
 
         Returns
         -------
@@ -232,9 +244,10 @@ class Cone(Shape):
 
         vertices = [[0, 0, 0]]
         a = 2 * pi / u
+        radius = self.circle.radius
         for i in range(u):
-            x = self.circle.radius * cos(i * a)
-            y = self.circle.radius * sin(i * a)
+            x = radius * cos(i * a)
+            y = radius * sin(i * a)
             vertices.append([x, y, 0])
         vertices.append([0, 0, self.height])
 
@@ -251,6 +264,16 @@ class Cone(Shape):
         faces.append([last - 1, 1, last])
         faces.append([1, last - 1, first])
 
+        if triangulated:
+            triangles = []
+            for face in faces:
+                if len(face) == 4:
+                    triangles.append(face[0:3])
+                    triangles.append([face[0], face[2], face[3]])
+                else:
+                    triangles.append(face)
+            faces = triangles
+
         return vertices, faces
 
     def transform(self, transformation):
